package loaders

import org.briljantframework.data.dataframe.DataFrame
import org.briljantframework.data.dataseries.DataSeriesCollection
import org.briljantframework.data.vector.DoubleVector
import org.briljantframework.data.vector.Vector
import org.briljantframework.data.vector.Vectors
import org.briljantframework.data.reader.StringDataEntry


/**
 * Created by isak on 04/09/15.
 */
class MatlabExtractLoader {

  String dir
  Tuple load() {
    def x = new DataSeriesCollection.Builder(Vector)
    def y = Vector.Builder.of(Integer)
    def instances = new File("$dir${File.separator}/train/data")
    def classes = new File("$dir${File.separator}/train/classes.dat")
    readData(instances, classes, x, y)

    instances = new File("$dir${File.separator}/test/data")
    classes = new File("$dir${File.separator}/test/classes.dat")
    if (instances.exists()) {
      readData(instances, classes, x, y)
    }

    assert y.size() == x.rows()
    return [y.build(), x.build()]
  }

  def static readData(File instances, File classes, DataFrame.Builder x, Vector.Builder y) {
    for (String name in instances.listFiles().collect({basename(it.name) as int}).sort()) {
      def file = new File("${instances.absolutePath}${File.separator}${name}.dat")
      List<Vector.Builder> channels = []
      file.eachLine {line ->
        def entry = new StringDataEntry(line.split(","))
        channels << Vector.Builder.of(Double).readAll(entry)
      }
      def v = Vector.Builder.of(Vector).addAll(channels.collect {it.build()})
      x.addRecord(v)
    }

    y.readAll(new StringDataEntry(classes.readLines().first().split(",")))
  }

  def static String basename(String name) {
    return name.lastIndexOf('.').with {it != -1 ? name[0..<it] : name}
  }
}
